package com.sxtvs.open.api.clue.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sxtvs.open.api.clue.dto.ClueDTO;
import com.sxtvs.open.api.clue.dto.VerifyDTO;
import com.sxtvs.open.api.clue.entity.Clue;
import com.sxtvs.open.api.clue.entity.ClueInfo;
import com.sxtvs.open.api.clue.entity.ClueReply;
import com.sxtvs.open.api.clue.entity.Info;
import com.sxtvs.open.api.clue.mapper.ClueMapper;
import com.sxtvs.open.api.clue.service.IClueService;
import com.sxtvs.open.core.advice.BizException;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.http.util.TextUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-03-20
 */
@Service
public class ClueServiceImpl extends ServiceImpl<ClueMapper, Clue> implements IClueService {

    @Resource
    private ClueReplyServiceImpl clueReplyService;

    @Resource
    private ClueInfoServiceImpl clueInfoService;

    public IPage<Clue> cluePage(ClueDTO clueDTO){

        LambdaQueryWrapper<Clue> wrapper = Wrappers.lambdaQuery();

        IPage<Clue> clueIPage = baseMapper.selectPage(new Page<>(clueDTO.getPage(), clueDTO.getSize()),
                wrapper.ge(clueDTO.getStart() != null, Clue::getAddTime, clueDTO.getStart())
                .lt(clueDTO.getEnd() != null, Clue::getAddTime, clueDTO.getEnd())
                .like(!TextUtils.isEmpty(clueDTO.getTitle()), Clue::getTitle, "%" + clueDTO.getTitle() + "%")
                .eq(Clue::getValidity, (byte) 1)
                .orderByDesc(Clue::getAddTime)
        );

        Map<Long, Pair<String, Integer>> collect = clueInfoService.lambdaQuery().in(
                ClueInfo::getRelaId, clueIPage.getRecords().stream().map(Clue::getId).collect(Collectors.toList())
        ).list().stream().collect(Collectors.groupingBy(ClueInfo::getRelaId, Collectors.collectingAndThen(Collectors.toList(),
                data -> new ImmutablePair<>(Strings.join(data.stream().map(ClueInfo::getType).distinct().collect(Collectors.toList()), ','), data.size())
        )));

        clueIPage.setRecords(clueIPage.getRecords().stream().peek(v -> {
            v.setClueInfoCount(collect.containsKey(v.getId()) ? collect.get(v.getId()).getRight() : 1);
            v.setClueInfoType(collect.containsKey(v.getId()) ? collect.get(v.getId()).getLeft() : "0");
        }).collect(Collectors.toList()));

        return clueIPage;
    }

    public void store(Clue clue){
        clue.setId(null);
        clue.setAddTime(LocalDateTime.now());
        clue.setCreateTime(LocalDateTime.now());
        clue.setModifyTime(LocalDateTime.now());
        save(clue);
    }

    public void verify(VerifyDTO verifyDTO){
        Clue clue = getById(verifyDTO.getId());
        if(clue.getStatus() != 0){
            throw new BizException(40006, "该条数据已审核");
        }
        clue.setStatus(verifyDTO.getStatus());
        updateById(clue);
    }

    @Transactional
    public void reply(ClueReply clueReply){
        Clue clue = getById(clueReply.getClueId());
        clue.setReplyFlag(1);
        clue.setReplyTime(LocalDateTime.now());
        updateById(clue);
        clueReply.setId(null);
        clueReply.setReplyTime(LocalDateTime.now());
        clueReply.setModifyTime(LocalDateTime.now());
        clueReplyService.save(clueReply);
    }

    @Transactional
    public void delete(Long id){
        Clue clue = getById(id);
        if(null == clue){
            throw new BizException(40006, "不存在的信息");
        }
        clue.setValidity((byte) 0);
        updateById(clue);
        clueReplyService.updateBatchById(
                clueReplyService.lambdaQuery().eq(ClueReply::getClueId, clue.getId()).list().stream()
                        .peek(data -> data.setStatus(-1)).collect(Collectors.toList())
        );
    }

    public Info info(Long id){
        Clue clue = getById(id);
        Info info = new Info();
        BeanUtil.copyProperties(clue, info);
        info.setClueReplies(clueReplyService.lambdaQuery().eq(ClueReply::getClueId,clue.getId()).list());
        info.setClueInfos(clueInfoService.lambdaQuery().eq(ClueInfo::getRelaId, clue.getId()).list());
        return info;
    }

}
